@@ -38,7 +38,8 @@ DIFF_MATCH_PATCH_REQVER = '>=20181111'
 # None for pynsist install for now
 # (check way to add dist.info/egg.info from packages without wheels available)
 INTERVALTREE_REQVER = None if is_pynsist() else '>=3.0.2'
-IPYTHON_REQVER = ">=7.31.1,<9.0.0,!=8.8.0,!=8.9.0,!=8.10.0"
+IPYTHON_REQVER = (
+    ">=7.31.1,<9.0.0,!=8.8.0,!=8.9.0,!=8.10.0,!=8.11.0,!=8.12.0,!=8.12.1")
 JEDI_REQVER = '>=0.17.2,<0.19.0'
 JELLYFISH_REQVER = '>=0.7'
 JSONSCHEMA_REQVER = '>=3.2.0'
@@ -52,9 +53,9 @@ PICKLESHARE_REQVER = '>=0.4'
 PSUTIL_REQVER = '>=5.3'
 PYGMENTS_REQVER = '>=2.0'
 PYLINT_REQVER = '>=2.5.0,<3.0'
-PYLINT_VENV_REQVER = '>=2.1.1'
-PYLSP_REQVER = '>=1.7.2,<1.8.0'
-PYLSP_BLACK_REQVER = '>=1.2.0'
+PYLINT_VENV_REQVER = '>=3.0.2'
+PYLSP_REQVER = '>=1.7.4,<1.8.0'
+PYLSP_BLACK_REQVER = '>=1.2.0,<3.0.0'
 PYLS_SPYDER_REQVER = '>=0.4.0'
 PYXDG_REQVER = '>=0.26'
 PYZMQ_REQVER = '>=22.1.0'
@@ -66,7 +67,7 @@ QTPY_REQVER = '>=2.1.0'
 RTREE_REQVER = '>=0.9.7'
 SETUPTOOLS_REQVER = '>=49.6.0'
 SPHINX_REQVER = '>=0.6.6'
-SPYDER_KERNELS_REQVER = '>=2.4.3,<2.5.0'
+SPYDER_KERNELS_REQVER = '>=2.4.4,<2.5.0'
 TEXTDISTANCE_REQVER = '>=4.2.0'
 THREE_MERGE_REQVER = '>=0.1.1'
 # None for pynsist install for now
