@@ -12,6 +12,7 @@ Holds references for base actions in the Application of Spyder.
 
 # Standard library imports
 import os
+import subprocess
 import sys
 import glob
 
@@ -21,14 +22,17 @@ from qtpy.QtGui import QGuiApplication
 from qtpy.QtWidgets import QAction, QMessageBox, QPushButton
 
 # Local imports
-from spyder import __docs_url__, __forum_url__, __trouble_url__
+from spyder import __docs_url__, __forum_url__, __trouble_url__, __version__
 from spyder import dependencies
-from spyder.api.translations import get_translation
+from spyder.api.translations import _
 from spyder.api.widgets.main_container import PluginMainContainer
 from spyder.utils.installers import InstallerMissingDependencies
 from spyder.config.utils import is_anaconda
-from spyder.config.base import get_conf_path, get_debug_level
+from spyder.config.base import (get_conf_path, get_debug_level, is_pynsist,
+                                running_in_mac_app)
+from spyder.plugins.application.widgets.status import ApplicationUpdateStatus
 from spyder.plugins.console.api import ConsoleActions
+from spyder.utils.environ import UserEnvDialog
 from spyder.utils.qthelpers import start_file, DialogManager
 from spyder.widgets.about import AboutDialog
 from spyder.widgets.dependencies import DependenciesDialog
@@ -36,14 +40,6 @@ from spyder.widgets.helperwidgets import MessageCheckBox
 from spyder.workers.updates import WorkerUpdates
 
 
-WinUserEnvDialog = None
-if os.name == 'nt':
-    from spyder.utils.environ import WinUserEnvDialog
-
-# Localization
-_ = get_translation('spyder')
-
-
 class ApplicationPluginMenus:
     DebugLogsMenu = "debug_logs_menu"
 
@@ -68,7 +64,7 @@ class ApplicationActions:
     SpyderAbout = "spyder_about_action"
 
     # Tools
-    SpyderWindowsEnvVariables = "spyder_windows_env_variables_action"
+    SpyderUserEnvVariables = "spyder_user_env_variables_action"
 
     # File
     # The name of the action needs to match the name of the shortcut
@@ -96,14 +92,27 @@ class ApplicationContainer(PluginMainContainer):
         self.current_dpi = None
         self.dpi_messagebox = None
 
+        # Keep track of the downloaded installer executable for updates
+        self.installer_path = None
+
     # ---- PluginMainContainer API
     # -------------------------------------------------------------------------
     def setup(self):
+
         # Compute dependencies in a thread to not block the interface.
         self.dependencies_thread = QThread(None)
 
         # Attributes
         self.dialog_manager = DialogManager()
+        self.application_update_status = None
+        if is_pynsist() or running_in_mac_app():
+            self.application_update_status = ApplicationUpdateStatus(
+                parent=self)
+            (self.application_update_status.sig_check_for_updates_requested
+             .connect(self.check_updates))
+            (self.application_update_status.sig_install_on_close_requested
+                 .connect(self.set_installer_path))
+            self.application_update_status.set_no_status()
         self.give_updates_feedback = False
         self.thread_updates = None
         self.worker_updates = None
@@ -161,17 +170,18 @@ class ApplicationContainer(PluginMainContainer):
             menurole=QAction.AboutRole)
 
         # Tools actions
-        if WinUserEnvDialog is not None:
-            self.winenv_action = self.create_action(
-                ApplicationActions.SpyderWindowsEnvVariables,
-                _("Current user environment variables..."),
-                icon=self.create_icon('win_env'),
-                tip=_("Show and edit current user environment "
-                      "variables in Windows registry "
-                      "(i.e. for all sessions)"),
-                triggered=self.show_windows_env_variables)
+        if os.name == 'nt':
+            tip = _("Show and edit current user environment variables in "
+                    "Windows registry (i.e. for all sessions)")
         else:
-            self.winenv_action = None
+            tip = _("Show current user environment variables (i.e. for all "
+                    "sessions)")
+        self.user_env_action = self.create_action(
+            ApplicationActions.SpyderUserEnvVariables,
+            _("Current user environment variables..."),
+            icon=self.create_icon('environment'),
+            tip=tip,
+            triggered=self.show_user_env_variables)
 
         # Application base actions
         self.restart_action = self.create_action(
@@ -222,6 +232,11 @@ class ApplicationContainer(PluginMainContainer):
             self.dependencies_thread.quit()
             self.dependencies_thread.wait()
 
+        # Run installer after Spyder is closed
+        cmd = ('start' if os.name == 'nt' else 'open')
+        if self.installer_path:
+            subprocess.Popen(' '.join([cmd, self.installer_path]), shell=True)
+
     @Slot()
     def show_about(self):
         """Show Spyder About dialog."""
@@ -229,12 +244,13 @@ class ApplicationContainer(PluginMainContainer):
         abt.show()
 
     @Slot()
-    def show_windows_env_variables(self):
+    def show_user_env_variables(self):
         """Show Windows current user environment variables."""
-        self.dialog_manager.show(WinUserEnvDialog(self))
+        self.dialog_manager.show(UserEnvDialog(self))
 
     # ---- Updates
     # -------------------------------------------------------------------------
+
     def _check_updates_ready(self):
         """Show results of the Spyder update checking process."""
 
@@ -279,10 +295,16 @@ class ApplicationContainer(PluginMainContainer):
             check_updates = box.is_checked()
         else:
             if update_available:
-                header = _("<b>Spyder {} is available!</b><br><br>").format(
-                    latest_release)
+                if self.application_update_status:
+                    self.application_update_status.set_status_pending(
+                        latest_release=latest_release)
+
+                header = _("<b>Spyder {} is available!</b> "
+                           "<i>(you have {})</i><br><br>").format(
+                    latest_release, __version__)
+                content = ""
                 footer = _(
-                    "For more information visit our "
+                    "For more information, visit our "
                     "<a href=\"{}\">installation guide</a>."
                 ).format(url_i)
                 if is_anaconda():
@@ -296,15 +318,25 @@ class ApplicationContainer(PluginMainContainer):
                         "<code>conda update anaconda</code><br>"
                         "<code>conda install spyder={}</code><br><br>"
                     ).format(latest_release)
-                else:
+                elif is_pynsist() or running_in_mac_app():
+                    box.setStandardButtons(QMessageBox.Yes |
+                                           QMessageBox.No)
                     content = _(
-                        "Click <a href=\"{}\">this link</a> to "
-                        "download it.<br><br>"
-                    ).format(url_r)
+                        "Would you like to automatically download and "
+                        "install it?<br><br>"
+                    )
+
                 msg = header + content + footer
                 box.setText(msg)
                 box.set_check_visible(True)
-                box.show()
+                box.exec_()
+                if self.application_update_status:
+                    if box.result() == QMessageBox.Yes:
+                        self.application_update_status.start_installation(
+                            latest_release=latest_release)
+                    elif box.result() == QMessageBox.No:
+                        self.application_update_status.set_status_pending(
+                            latest_release=latest_release)
                 check_updates = box.is_checked()
             elif feedback:
                 msg = _("Spyder is up to date.")
@@ -312,7 +344,11 @@ class ApplicationContainer(PluginMainContainer):
                 box.set_check_visible(False)
                 box.exec_()
                 check_updates = box.is_checked()
-
+                if self.application_update_status:
+                    self.application_update_status.set_no_status()
+            else:
+                if self.application_update_status:
+                    self.application_update_status.set_no_status()
         # Update checkbox based on user interaction
         self.set_conf(option, check_updates)
 
@@ -327,6 +363,8 @@ class ApplicationContainer(PluginMainContainer):
         """Check for spyder updates on github releases using a QThread."""
         # Disable check_updates_action while the thread is working
         self.check_updates_action.setDisabled(True)
+        if self.application_update_status:
+            self.application_update_status.set_status_checking()
 
         if self.thread_updates is not None:
             self.thread_updates.quit()
@@ -343,11 +381,16 @@ class ApplicationContainer(PluginMainContainer):
         # while loading.
         # Fixes spyder-ide/spyder#15839
         self.updates_timer = QTimer(self)
-        self.updates_timer.setInterval(3000)
+        self.updates_timer.setInterval(60000)
         self.updates_timer.setSingleShot(True)
         self.updates_timer.timeout.connect(self.thread_updates.start)
         self.updates_timer.start()
 
+    @Slot(str)
+    def set_installer_path(self, installer_path):
+        """Set installer executable path to be run when closing."""
+        self.installer_path = installer_path
+
     # ---- Dependencies
     # -------------------------------------------------------------------------
     @Slot()
@@ -549,7 +592,12 @@ class ApplicationContainer(PluginMainContainer):
                     sys.platform == 'darwin'):
                 return
 
+            if self.get_conf('high_dpi_scaling'):
+                return
+
             if self.dpi_messagebox is not None:
+                self.dpi_messagebox.activateWindow()
+                self.dpi_messagebox.raise_()
                 return
 
             self.dpi_messagebox = MessageCheckBox(icon=QMessageBox.Warning,
@@ -577,6 +625,7 @@ class ApplicationContainer(PluginMainContainer):
             self.dpi_messagebox.finished.connect(
                 lambda result: self.handle_dpi_change_response(result, dpi))
             self.dpi_messagebox.open()
+
             # Show dialog always in the primary screen to prevent not being
             # able to see it if a screen gets disconnected while
             # in suspended state. See spyder-ide/spyder#16390
@@ -585,4 +634,8 @@ class ApplicationContainer(PluginMainContainer):
             screen_geometry = QGuiApplication.primaryScreen().geometry()
             x = (screen_geometry.width() - dpi_messagebox_width) / 2
             y = (screen_geometry.height() - dpi_messagebox_height) / 2
-            self.dpi_messagebox.move(x, y)
+
+            # Convert coordinates to int to avoid a TypeError in Python 3.10
+            # Fixes spyder-ide/spyder#17677
+            self.dpi_messagebox.move(int(x), int(y))
+            self.dpi_messagebox.adjustSize()
