@@ -19,14 +19,12 @@ from qtpy.QtGui import QIcon
 from spyder.api.plugins import Plugins, SpyderDockablePlugin
 from spyder.api.plugin_registration.decorators import (
     on_plugin_available, on_plugin_teardown)
-from spyder.api.translations import get_translation
+from spyder.api.translations import _
 from spyder.config.base import DEV
 from spyder.plugins.console.widgets.main_widget import (
     ConsoleWidget, ConsoleWidgetActions)
 from spyder.plugins.mainmenu.api import ApplicationMenus, FileMenuSections
 
-# Localization
-_ = get_translation('spyder')
 
 # Logging
 logger = logging.getLogger(__name__)
@@ -95,7 +93,6 @@ class Console(SpyderDockablePlugin):
         # Signals
         widget.sig_edit_goto_requested.connect(self.sig_edit_goto_requested)
         widget.sig_focus_changed.connect(self.sig_focus_changed)
-        widget.sig_quit_requested.connect(self.sig_quit_requested)
         widget.sig_refreshed.connect(self.sig_refreshed)
         widget.sig_help_requested.connect(self.sig_help_requested)
 
@@ -260,7 +257,6 @@ class Console(SpyderDockablePlugin):
         Stdin and stdout are now redirected through the internal console.
         """
         widget = self.get_widget()
-        widget.set_conf('namespace', namespace)
         widget.start_interpreter(namespace)
 
     def set_namespace_item(self, name, value):
