@@ -9,6 +9,7 @@ IPython Console main widget based on QtConsole.
 """
 
 # Standard library imports
+import logging
 import os
 import os.path as osp
 import sys
@@ -18,6 +19,7 @@ import uuid
 # Third-party imports
 from jupyter_client.connect import find_connection_file
 from jupyter_core.paths import jupyter_config_dir, jupyter_runtime_dir
+import qstylizer.style
 from qtconsole.client import QtKernelClient
 from qtpy.QtCore import Signal, Slot
 from qtpy.QtGui import QColor
@@ -29,7 +31,7 @@ from zmq.ssh import tunnel as zmqtunnel
 
 # Local imports
 from spyder.api.config.decorators import on_conf_change
-from spyder.api.translations import get_translation
+from spyder.api.translations import _
 from spyder.api.widgets.main_widget import PluginMainWidget
 from spyder.api.widgets.menus import MENU_SEPARATOR
 from spyder.config.base import (
@@ -41,7 +43,6 @@ from spyder.plugins.ipythonconsole.utils.style import create_qss_style
 from spyder.plugins.ipythonconsole.widgets import (
     ClientWidget, ConsoleRestartDialog, COMPLETION_WIDGET_TYPE,
     KernelConnectionDialog, PageControlWidget, ShellWidget)
-from spyder.py3compat import PY38_OR_MORE
 from spyder.utils import encoding, programs, sourcecode
 from spyder.utils.misc import get_error_match, remove_backslashes
 from spyder.utils.palette import QStylePalette
@@ -51,24 +52,14 @@ from spyder.widgets.tabs import Tabs
 from spyder.plugins.ipythonconsole.utils.stdfile import StdFile
 
 
-# Localization
-_ = get_translation('spyder')
+# Logging
+logger = logging.getLogger(__name__)
+
 
 # =============================================================================
 # ---- Constants
 # =============================================================================
 MAIN_BG_COLOR = QStylePalette.COLOR_BACKGROUND_1
-SPYDER_KERNELS_MIN_VERSION = '2.3.0'
-SPYDER_KERNELS_MAX_VERSION = '2.4.0'
-SPYDER_KERNELS_VERSION = (
-    f'>={SPYDER_KERNELS_MIN_VERSION};<{SPYDER_KERNELS_MAX_VERSION}')
-SPYDER_KERNELS_VERSION_MSG = _(
-    '>= {0} and < {1}').format(
-        SPYDER_KERNELS_MIN_VERSION, SPYDER_KERNELS_MAX_VERSION)
-SPYDER_KERNELS_CONDA = (
-    f'conda install spyder&#45;kernels={SPYDER_KERNELS_MIN_VERSION[:-2]}')
-SPYDER_KERNELS_PIP = (
-    f'pip install spyder&#45;kernels=={SPYDER_KERNELS_MIN_VERSION[:-1]}*')
 
 
 class IPythonConsoleWidgetActions:
@@ -109,16 +100,21 @@ class IPythonConsoleWidgetOptionsMenus:
     Documentation = 'documentation_submenu'
 
 
-class IPythonConsoleWidgetConsolesMenusSection:
-    Main = 'main_section'
-
-
 class IPythonConsoleWidgetOptionsMenuSections:
     Consoles = 'consoles_section'
     Edit = 'edit_section'
     View = 'view_section'
 
 
+class IPythonConsoleWidgetMenus:
+    TabsContextMenu = 'tabs_context_menu'
+
+
+class IPythonConsoleWidgetTabsContextMenuSections:
+    Consoles = 'tabs_consoles_section'
+    Edit = 'tabs_edit_section'
+
+
 # --- Widgets
 # ----------------------------------------------------------------------------
 class IPythonConsoleWidget(PluginMainWidget):
@@ -327,10 +323,9 @@ class IPythonConsoleWidget(PluginMainWidget):
 
         layout = QVBoxLayout()
         layout.setSpacing(0)
-        self.tabwidget = Tabs(self, menu=self._options_menu,
-                              actions=self.menu_actions,
-                              rename_tabs=True,
-                              split_char='/', split_index=0)
+
+        self.tabwidget = Tabs(self, rename_tabs=True, split_char='/',
+                              split_index=0)
         if (hasattr(self.tabwidget, 'setDocumentMode')
                 and not sys.platform == 'darwin'):
             # Don't set document mode to true on OSX because it generates
@@ -369,13 +364,15 @@ class IPythonConsoleWidget(PluginMainWidget):
 
         # Label to inform users how to get out of the pager
         self.pager_label = QLabel(_("Press <b>Q</b> to exit pager"), self)
-        self.pager_label.setStyleSheet(
-            f"background-color: {QStylePalette.COLOR_ACCENT_2};"
-            f"color: {QStylePalette.COLOR_TEXT_1};"
-            "margin: 0px 1px 4px 1px;"
-            "padding: 5px;"
-            "qproperty-alignment: AlignCenter;"
-        )
+        pager_label_css = qstylizer.style.StyleSheet()
+        pager_label_css.setValues(**{
+            'background-color': f'{QStylePalette.COLOR_ACCENT_2}',
+            'color': f'{QStylePalette.COLOR_TEXT_1}',
+            'margin': '0px 1px 4px 1px',
+            'padding': '5px',
+            'qproperty-alignment': 'AlignCenter'
+        })
+        self.pager_label.setStyleSheet(pager_label_css.toString())
         self.pager_label.hide()
         layout.addWidget(self.pager_label)
 
@@ -401,7 +398,7 @@ class IPythonConsoleWidget(PluginMainWidget):
 
     def on_close(self):
         self.mainwindow_close = True
-        self.close_clients()
+        self.close_all_clients()
 
     # ---- PluginMainWidget API and settings handling
     # ------------------------------------------------------------------------
@@ -414,7 +411,7 @@ class IPythonConsoleWidget(PluginMainWidget):
             return client.get_control()
 
     def setup(self):
-        # Options menu actions
+        # --- Options menu actions
         self.create_client_action = self.create_action(
             IPythonConsoleWidgetActions.CreateNewClient,
             text=_("New console (default settings)"),
@@ -456,7 +453,7 @@ class IPythonConsoleWidget(PluginMainWidget):
             triggered=self.tab_name_editor,
         )
 
-        # For the client
+        # --- For the client
         self.env_action = self.create_action(
             IPythonConsoleWidgetActions.ShowEnvironmentVariables,
             text=_("Show environment variables"),
@@ -482,7 +479,7 @@ class IPythonConsoleWidget(PluginMainWidget):
             initial=self.get_conf('show_elapsed_time')
         )
 
-        # Context menu actions
+        # --- Context menu actions
         # TODO: Shortcut registration not working
         self.inspect_action = self.create_action(
             IPythonConsoleWidgetActions.InspectObject,
@@ -509,7 +506,7 @@ class IPythonConsoleWidget(PluginMainWidget):
             icon=self.create_icon('exit'),
             triggered=self.current_client_quit)
 
-        # Other actions with shortcuts
+        # --- Other actions with shortcuts
         self.array_table_action = self.create_action(
             IPythonConsoleWidgetActions.ArrayTable,
             text=_("Enter array table"),
@@ -534,6 +531,7 @@ class IPythonConsoleWidget(PluginMainWidget):
             self.quit_action
         )
 
+        # --- Setting options menu
         options_menu = self.get_options_menu()
         self.special_console_menu = self.create_menu(
             IPythonConsoleWidgetOptionsMenus.SpecialConsoles,
@@ -595,11 +593,10 @@ class IPythonConsoleWidget(PluginMainWidget):
                 create_cython_action]:
             self.add_item_to_menu(
                 item,
-                menu=self.special_console_menu,
-                section=IPythonConsoleWidgetConsolesMenusSection.Main,
+                menu=self.special_console_menu
             )
 
-        # Widgets for the tab corner
+        # --- Widgets for the tab corner
         self.reset_button = self.create_toolbutton(
             'reset',
             text=_("Remove all variables"),
@@ -616,12 +613,39 @@ class IPythonConsoleWidget(PluginMainWidget):
         )
         self.time_label = QLabel("")
 
-        # Add tab corner widgets.
+        # --- Add tab corner widgets.
+        self.add_corner_widget('timer', self.time_label)
         self.add_corner_widget('reset', self.reset_button)
         self.add_corner_widget('start_interrupt', self.stop_button)
-        self.add_corner_widget('timer', self.time_label)
 
-        # Create IPython documentation menu
+        # --- Tabs context menu
+        tabs_context_menu = self.create_menu(
+            IPythonConsoleWidgetMenus.TabsContextMenu)
+
+        for item in [
+                self.create_client_action,
+                self.special_console_menu,
+                self.connect_to_kernel_action]:
+            self.add_item_to_menu(
+                item,
+                menu=tabs_context_menu,
+                section=IPythonConsoleWidgetTabsContextMenuSections.Consoles,
+            )
+
+        for item in [
+                self.interrupt_action,
+                self.restart_action,
+                self.reset_action,
+                self.rename_tab_action]:
+            self.add_item_to_menu(
+                item,
+                menu=tabs_context_menu,
+                section=IPythonConsoleWidgetTabsContextMenuSections.Edit,
+            )
+
+        self.tabwidget.menu = tabs_context_menu
+
+        # --- Create IPython documentation menu
         self.ipython_menu = self.create_menu(
             menu_id=IPythonConsoleWidgetOptionsMenus.Documentation,
             title=_("IPython documentation"))
@@ -654,10 +678,23 @@ class IPythonConsoleWidget(PluginMainWidget):
     def update_actions(self):
         client = self.get_current_client()
         if client is not None:
+            # Executing state
             executing = client.is_client_executing()
             self.interrupt_action.setEnabled(executing)
             self.stop_button.setEnabled(executing)
 
+            # Client is loading or showing a kernel error
+            if (
+                client.infowidget is not None
+                and client.info_page is not None
+            ):
+                error_or_loading = client.info_page != client.blank_page
+                self.restart_action.setEnabled(not error_or_loading)
+                self.reset_action.setEnabled(not error_or_loading)
+                self.env_action.setEnabled(not error_or_loading)
+                self.syspath_action.setEnabled(not error_or_loading)
+                self.show_time_action.setEnabled(not error_or_loading)
+
     # ---- GUI options
     @on_conf_change(section='help', option='connect/ipython_console')
     def change_clients_help_connection(self, value):
@@ -667,8 +704,11 @@ class IPythonConsoleWidget(PluginMainWidget):
                 client.get_control().set_help_enabled,
                 value)
 
-    @on_conf_change(section='appearance', option='selected')
-    def change_clients_color_scheme(self, value):
+    @on_conf_change(section='appearance', option=['selected', 'ui_theme'])
+    def change_clients_color_scheme(self, option, value):
+        if option == 'ui_theme':
+            value = self.get_conf('selected', section='appearance')
+
         for idx, client in enumerate(self.clients):
             self._change_client_conf(
                 client,
@@ -1026,7 +1066,7 @@ class IPythonConsoleWidget(PluginMainWidget):
         - Do this as early as possible to make it a low priority and
           overrideable.
         """
-        if os.name == 'nt' and PY38_OR_MORE:
+        if os.name == 'nt' and sys.version_info[:2] >= (3, 8):
             # Tests on Linux hang if we don't leave this import here.
             import tornado
             if tornado.version_info >= (6, 1):
@@ -1161,6 +1201,14 @@ class IPythonConsoleWidget(PluginMainWidget):
                 sw.is_waiting_pdb_input(), sw.get_pdb_last_step())
             self.sig_shellwidget_changed.emit(sw)
 
+            # This is necessary to sync the current client cwd with the working
+            # directory displayed by other plugins in Spyder (e.g. Files).
+            # NOTE: Instead of emitting sig_current_directory_changed directly,
+            # we call on_working_directory_changed to validate that the cwd
+            # exists (this couldn't be the case for remote kernels).
+            if sw.get_cwd() != self.get_working_directory():
+                self.on_working_directory_changed(sw.get_cwd())
+
         self.update_tabs_text()
         self.update_actions()
 
@@ -1409,7 +1457,7 @@ class IPythonConsoleWidget(PluginMainWidget):
     @Slot(bool, str, bool)
     def create_new_client(self, give_focus=True, filename='', is_cython=False,
                           is_pylab=False, is_sympy=False, given_name=None,
-                          cache=True):
+                          cache=True, initial_cwd=None):
         """Create a new client"""
         self.master_clients += 1
         client_id = dict(int_id=str(self.master_clients),
@@ -1436,7 +1484,6 @@ class IPythonConsoleWidget(PluginMainWidget):
                               interpreter_versions=self.interpreter_versions(),
                               connection_file=cf,
                               context_menu_actions=self.context_menu_actions,
-                              time_label=self.time_label,
                               show_elapsed_time=show_elapsed_time,
                               reset_warning=reset_warning,
                               given_name=given_name,
@@ -1447,7 +1494,8 @@ class IPythonConsoleWidget(PluginMainWidget):
                               handlers=self.registered_spyder_kernel_handlers,
                               stderr_obj=stderr_obj,
                               stdout_obj=stdout_obj,
-                              fault_obj=fault_obj)
+                              fault_obj=fault_obj,
+                              initial_cwd=initial_cwd)
 
         self.add_tab(
             client, name=client.get_name(), filename=filename,
@@ -1460,44 +1508,14 @@ class IPythonConsoleWidget(PluginMainWidget):
 
         # Check if ipykernel is present in the external interpreter.
         # Else we won't be able to create a client
-        if not self.get_conf('default', section='main_interpreter'):
-            pyexec = self.get_conf('executable', section='main_interpreter')
-            has_spyder_kernels = programs.is_module_installed(
-                'spyder_kernels',
-                interpreter=pyexec,
-                version=SPYDER_KERNELS_VERSION)
-            if not has_spyder_kernels and not running_under_pytest():
-                client.show_kernel_error(
-                    _("The Python environment or installation whose "
-                      "interpreter is located at"
-                      "<pre>"
-                      "    <tt>{0}</tt>"
-                      "</pre>"
-                      "doesn't have the <tt>spyder-kernels</tt> module or the "
-                      "right version of it installed ({1}). "
-                      "Without this module is not possible for Spyder to "
-                      "create a console for you.<br><br>"
-                      "You can install it by activating your environment (if "
-                      "necessary) and then running in a system terminal:"
-                      "<pre>"
-                      "    <tt>{2}</tt>"
-                      "</pre>"
-                      "or"
-                      "<pre>"
-                      "    <tt>{3}</tt>"
-                      "</pre>").format(
-                          pyexec,
-                          SPYDER_KERNELS_VERSION_MSG,
-                          SPYDER_KERNELS_CONDA,
-                          SPYDER_KERNELS_PIP
-                      )
-                )
-                return
+        if not client.has_spyder_kernels():
+            return
 
         self.connect_client_to_kernel(client, km, kc)
         if client.shellwidget.kernel_manager is None:
             return
         self.register_client(client, give_focus=give_focus)
+        return client
 
     def create_client_for_kernel(self, connection_file, hostname, sshkey,
                                  password):
@@ -1575,7 +1593,6 @@ class IPythonConsoleWidget(PluginMainWidget):
                               interpreter_versions=self.interpreter_versions(),
                               connection_file=connection_file,
                               context_menu_actions=self.context_menu_actions,
-                              time_label=self.time_label,
                               hostname=hostname,
                               is_external_kernel=is_external_kernel,
                               is_spyder_kernel=known_spyder_kernel,
@@ -1790,28 +1807,27 @@ class IPythonConsoleWidget(PluginMainWidget):
         # type is Qt.DirectConnection.
         self._shellwidget_started(client)
 
-
     @Slot(str)
     def create_client_from_path(self, path):
         """Create a client with its cwd pointing to path."""
-        self.create_new_client()
-        sw = self.get_current_shellwidget()
-        sw.set_cwd(path)
+        self.create_new_client(initial_cwd=path)
 
     def create_client_for_file(self, filename, is_cython=False):
         """Create a client to execute code related to a file."""
         # Create client
-        self.create_new_client(filename=filename, is_cython=is_cython)
+        client = self.create_new_client(
+            filename=filename, is_cython=is_cython)
 
         # Don't increase the count of master clients
         self.master_clients -= 1
 
         # Rename client tab with filename
-        client = self.get_current_client()
         client.allow_rename = False
         tab_text = self.disambiguate_fname(filename)
         self.rename_client_tab(client, tab_text)
 
+        return client
+
     def get_client_for_file(self, filename):
         """Get client associated with a given file."""
         client = None
@@ -1831,7 +1847,7 @@ class IPythonConsoleWidget(PluginMainWidget):
         control = shellwidget._control
 
         # Create new clients with Ctrl+T shortcut
-        shellwidget.new_client.connect(self.create_new_client)
+        shellwidget.sig_new_client.connect(self.create_new_client)
 
         # For tracebacks
         control.sig_go_to_error_requested.connect(self.go_to_error)
@@ -1860,11 +1876,14 @@ class IPythonConsoleWidget(PluginMainWidget):
 
         # Connect to working directory
         shellwidget.sig_working_directory_changed.connect(
-            self.working_directory_changed)
+            self.on_working_directory_changed)
 
         # Connect client execution state to be reflected in the interface
         client.sig_execution_state_changed.connect(self.update_actions)
 
+        # Show time label
+        client.sig_time_label.connect(self.time_label.setText)
+
     def close_client(self, index=None, client=None, ask_recursive=True):
         """Close client tab from index or widget (or close current tab)"""
         if not self.tabwidget.count():
@@ -1883,19 +1902,6 @@ class IPythonConsoleWidget(PluginMainWidget):
         if index is not None:
             client = self.tabwidget.widget(index)
 
-        # Needed to handle a RuntimeError. See spyder-ide/spyder#5568.
-        try:
-            # Close client
-            client.stop_button_click_handler()
-        except RuntimeError:
-            pass
-
-        # Disconnect timer needed to update elapsed time
-        try:
-            client.timer.timeout.disconnect(client.show_time)
-        except (RuntimeError, TypeError):
-            pass
-
         # Check if related clients or kernels are opened
         # and eventually ask before closing them
         if not self.mainwindow_close and ask_recursive:
@@ -1919,13 +1925,11 @@ class IPythonConsoleWidget(PluginMainWidget):
             if close_all == QMessageBox.Yes:
                 self.close_related_clients(client)
 
-        last_client = len(self.get_related_clients(client)) == 0
-        client.shutdown(last_client)
-        client.close()
-
         # Note: client index may have changed after closing related widgets
         self.tabwidget.removeTab(self.tabwidget.indexOf(client))
         self.clients.remove(client)
+        is_last_client = len(self.get_related_clients(client)) == 0
+        client.close_client(is_last_client)
 
         # This is needed to prevent that hanged consoles make reference
         # to an index that doesn't exist. See spyder-ide/spyder#4881
@@ -1940,7 +1944,7 @@ class IPythonConsoleWidget(PluginMainWidget):
         if not self.tabwidget.count() and self.create_new_client_if_empty:
             self.create_new_client()
 
-    def close_clients(self):
+    def close_all_clients(self):
         """
         Perform close actions for each running client.
 
@@ -1949,17 +1953,23 @@ class IPythonConsoleWidget(PluginMainWidget):
         bool
             If the closing action was succesful.
         """
+        # IMPORTANT: **Do not** change this way of closing clients, which uses
+        # a copy of `self.clients`, because it preserves the main window layout
+        # when Spyder is closed.
+        # Fixes spyder-ide/spyder#19084
         open_clients = self.clients.copy()
         for client in self.clients:
-            last_client = (
+            is_last_client = (
                 len(self.get_related_clients(client, open_clients)) == 0)
-            client.shutdown(last_client)
-            client.close()
+            client.close_client(is_last_client)
             open_clients.remove(client)
+
         # Close all closing shellwidgets.
         ShellWidget.wait_all_shutdown()
+
         # Close cached kernel
         self.close_cached_kernel()
+        self.filenames = []
         return True
 
     def get_client_index_from_id(self, client_id):
@@ -2177,25 +2187,30 @@ class IPythonConsoleWidget(PluginMainWidget):
         # editor during repeated, rapid entry of debugging commands.
         self.sig_edit_goto_requested[str, int, str, bool].emit(
             fname, lineno, '', False)
-        self.activateWindow()
-        shellwidget._control.setFocus()
+
+        # Give focus to console if requested
+        if shellwidget._pdb_focus_to_editor:
+            # Next focus to editor was enabled
+            shellwidget._pdb_focus_to_editor = False
+        else:
+            self.activateWindow()
+            shellwidget._control.setFocus()
 
     def set_spyder_breakpoints(self):
         """Set Spyder breakpoints into all clients"""
         for cl in self.clients:
             cl.shellwidget.set_spyder_breakpoints()
 
-    def pdb_execute_command(self, command):
+    def pdb_execute_command(self, command, focus_to_editor):
         """
         Send command to the pdb kernel if possible.
         """
-
         sw = self.get_current_shellwidget()
         if sw is not None:
             # Needed to handle an error when kernel_client is None.
             # See spyder-ide/spyder#7578.
             try:
-                sw.pdb_execute_command(command)
+                sw.pdb_execute_command(command, focus_to_editor)
             except AttributeError:
                 pass
 
@@ -2263,6 +2278,9 @@ class IPythonConsoleWidget(PluginMainWidget):
                                 repr(cell_name),
                                 norm(filename).replace("'", r"\'")))
 
+                if function == "debugcell":
+                    # To keep focus in editor after running debugfile
+                    client.shellwidget._pdb_focus_to_editor = focus_to_editor
             # External kernels and run_cell_copy, just execute the code
             else:
                 line = code.strip()
@@ -2274,7 +2292,7 @@ class IPythonConsoleWidget(PluginMainWidget):
 
             # This is necessary to prevent raising the console if the editor
             # and console are tabified next to each other and the 'Maintain
-            # focus in the editor' option is activated.
+            # focus in the editor' option is enabled.
             # Fixes spyder-ide/spyder#17028
             if not focus_to_editor:
                 self.sig_switch_to_plugin_requested.emit()
@@ -2297,7 +2315,8 @@ class IPythonConsoleWidget(PluginMainWidget):
 
     # ---- For scripts
     def run_script(self, filename, wdir, args, debug, post_mortem,
-                   current_client, clear_variables, console_namespace):
+                   current_client, clear_variables, console_namespace,
+                   focus_to_editor):
         """Run script in current or dedicated client"""
         norm = lambda text: remove_backslashes(str(text))
 
@@ -2313,8 +2332,8 @@ class IPythonConsoleWidget(PluginMainWidget):
         else:
             client = self.get_client_for_file(filename)
             if client is None:
-                self.create_client_for_file(filename, is_cython=is_cython)
-                client = self.get_current_client()
+                client = self.create_client_for_file(
+                    filename, is_cython=is_cython)
                 is_new_client = True
 
         if client is not None:
@@ -2331,6 +2350,10 @@ class IPythonConsoleWidget(PluginMainWidget):
                 if console_namespace:
                     line += ", current_namespace=True"
                 line += ")"
+
+                if debug:
+                    # To keep focus in editor after running debugfile
+                    client.shellwidget._pdb_focus_to_editor = focus_to_editor
             else:  # External, non spyder-kernels, use %run
                 line = "%run "
                 if debug:
@@ -2347,19 +2370,25 @@ class IPythonConsoleWidget(PluginMainWidget):
                     pass
                 elif current_client:
                     self.execute_code(line, current_client, clear_variables,
-                                      set_focus=False)
+                                      set_focus=not focus_to_editor)
                 else:
                     if is_new_client:
                         client.shellwidget.silent_execute('%clear')
                     else:
                         client.shellwidget.execute('%clear')
                     client.shellwidget.sig_prompt_ready.connect(
-                            lambda: self.execute_code(
-                                line, current_client, clear_variables,
-                                set_focus=False))
+                        lambda: self.execute_code(
+                            line, current_client, clear_variables,
+                            set_focus=not focus_to_editor,
+                            shellwidget=client.shellwidget
+                        )
+                    )
             except AttributeError:
                 pass
-            self.sig_switch_to_plugin_requested.emit()
+
+            # Show plugin after execution if focus is going to be given to it.
+            if not focus_to_editor:
+                self.sig_switch_to_plugin_requested.emit()
         else:
             # XXX: not sure it can really happen
             QMessageBox.warning(
@@ -2372,15 +2401,23 @@ class IPythonConsoleWidget(PluginMainWidget):
             )
 
     # ---- For working directory and path management
-    def set_working_directory(self, new_dir):
+    def set_working_directory(self, dirname):
         """
-        Set current working directory when changed by the Working Directory
+        Set current working directory in the Working Directory and Files
+        plugins.
+        """
+        if osp.isdir(dirname):
+            self.sig_current_directory_changed.emit(dirname)
+
+    def save_working_directory(self, dirname):
+        """
+        Save current working directory when changed by the Working Directory
         plugin.
         """
-        self._current_working_directory = new_dir
+        self._current_working_directory = dirname
 
     def get_working_directory(self):
-        """Get current working directory."""
+        """Get saved value of current working directory."""
         return self._current_working_directory
 
     def set_current_client_working_directory(self, directory):
@@ -2389,12 +2426,12 @@ class IPythonConsoleWidget(PluginMainWidget):
         if shellwidget is not None:
             shellwidget.set_cwd(directory)
 
-    def working_directory_changed(self, dirname):
+    def on_working_directory_changed(self, dirname):
         """
         Notify that the working directory was changed in the current console
         to other plugins.
         """
-        if osp.isdir(dirname):
+        if dirname and osp.isdir(dirname):
             self.sig_current_directory_changed.emit(dirname)
 
     def update_working_directory(self):
@@ -2405,6 +2442,7 @@ class IPythonConsoleWidget(PluginMainWidget):
 
     def update_path(self, path_dict, new_path_dict):
         """Update path on consoles."""
+        logger.debug("Update sys.path in all console clients")
         for client in self.clients:
             shell = client.shellwidget
             if shell is not None:
@@ -2432,9 +2470,12 @@ class IPythonConsoleWidget(PluginMainWidget):
 
     # ---- For execution
     def execute_code(self, lines, current_client=True, clear_variables=False,
-                     set_focus=True):
+                     set_focus=True, shellwidget=None):
         """Execute code instructions."""
-        sw = self.get_current_shellwidget()
+        if current_client:
+            sw = self.get_current_shellwidget()
+        else:
+            sw = shellwidget
         if sw is not None:
             if not current_client:
                 # Clear console and reset namespace for
@@ -2444,18 +2485,30 @@ class IPythonConsoleWidget(PluginMainWidget):
                     sw.sig_prompt_ready.disconnect()
                 except TypeError:
                     pass
-                sw.reset_namespace(warning=False)
+                if clear_variables:
+                    sw.reset_namespace(warning=False)
             elif current_client and clear_variables:
                 sw.reset_namespace(warning=False)
+
             # Needed to handle an error when kernel_client is none.
             # See spyder-ide/spyder#6308.
             try:
                 sw.execute(str(lines))
             except AttributeError:
                 pass
-            self.activateWindow()
+
             if set_focus:
-                self.get_current_client().get_control().setFocus()
+                # The `activateWindow` call below needs to be inside this `if`
+                # to avoid giving focus to the console when it's undocked,
+                # users are running code from the editor and the 'Maintain
+                # focus in the editor' option is enabled.
+                # Fixes spyder-ide/spyder#3221
+                self.activateWindow()
+
+                # Gives focus to the current client
+                focus_widget = self.get_focus_widget()
+                if focus_widget:
+                    focus_widget.setFocus()
 
     # ---- For error handling
     def go_to_error(self, text):
@@ -2463,9 +2516,17 @@ class IPythonConsoleWidget(PluginMainWidget):
         match = get_error_match(str(text))
         if match:
             fname, lnb = match.groups()
-            if ("<ipython-input-" in fname and
-                    self.run_cell_filename is not None):
+            if (
+                "<ipython-input-" in fname
+                and self.run_cell_filename is not None
+            ):
                 fname = self.run_cell_filename
+
+            # For IPython 8+ tracebacks.
+            # Fixes spyder-ide/spyder#20407
+            if '~' in fname:
+                fname = osp.expanduser(fname)
+
             # This is needed to fix issue spyder-ide/spyder#9217.
             try:
                 self.sig_edit_goto_requested.emit(
