@@ -105,13 +105,16 @@ class PythonpathContainer(PluginMainContainer):
 
     def show_path_manager(self):
         """Show path manager dialog."""
-        # Set main attributes saved here
-        self.path_manager_dialog.update_paths(
-            self.path, self.not_active_path, get_system_pythonpath()
-        )
+        # Do not update paths or run setup if widget is already open,
+        # see spyder-ide/spyder#20808
+        if not self.path_manager_dialog.isVisible():
+            # Set main attributes saved here
+            self.path_manager_dialog.update_paths(
+                self.path, self.not_active_path, get_system_pythonpath()
+            )
 
-        # Setup its contents again
-        self.path_manager_dialog.setup()
+            # Setup its contents again
+            self.path_manager_dialog.setup()
 
         # Show and give it focus
         self.path_manager_dialog.show()
