@@ -6,12 +6,8 @@
 
 """Module checking Spyder installation requirements"""
 
-# Standard library imports
-import sys
-import os.path as osp
-
 # Third-party imports
-from pkg_resources import parse_version
+from packaging.version import parse
 
 
 def show_warning(message):
@@ -29,19 +25,10 @@ def show_warning(message):
         root.mainloop()
     except Exception:
         pass
+
     raise RuntimeError(message)
 
 
-def check_path():
-    """Check sys.path: is Spyder properly installed?"""
-    dirname = osp.abspath(osp.join(osp.dirname(__file__), osp.pardir))
-    if dirname not in sys.path:
-        show_warning("Spyder must be installed properly "
-                     "(e.g. from source: 'python setup.py install'),\n"
-                     "or directory '%s' must be in PYTHONPATH "
-                     "environment variable." % dirname)
-
-
 def check_qt():
     """Check Qt binding requirements"""
     qt_infos = dict(pyqt5=("PyQt5", "5.9"), pyside2=("PySide2", "5.12"))
@@ -49,8 +36,11 @@ def check_qt():
         import qtpy
         package_name, required_ver = qt_infos[qtpy.API]
         actual_ver = qtpy.QT_VERSION
-        if (actual_ver is None or
-                parse_version(actual_ver) < parse_version(required_ver)):
+
+        if (
+            actual_ver is None
+            or parse(actual_ver) < parse(required_ver)
+        ):
             show_warning("Please check Spyder installation requirements:\n"
                          "%s %s+ is required (found %s)."
                          % (package_name, required_ver, actual_ver))
